@@ -19,6 +19,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <ctype.h>
 
 //----------------------------------------------------------------
 
@@ -69,7 +70,7 @@ struct node48 {
 };
 
 struct node256 {
-        uint32_t nr_entries;
+	uint32_t nr_entries;
 	struct value values[256];
 };
 
@@ -99,7 +100,7 @@ struct radix_tree *radix_tree_create(radix_value_dtr dtr, void *dtr_context)
 static inline void _dtr(struct radix_tree *rt, union radix_value v)
 {
 	if (rt->dtr)
-        	rt->dtr(rt->dtr_context, v);
+		rt->dtr(rt->dtr_context, v);
 }
 
 // Returns the number of values removed
@@ -118,8 +119,8 @@ static unsigned _free_node(struct radix_tree *rt, struct value v)
 		break;
 
 	case VALUE:
-        	_dtr(rt, v.value);
-        	nr = 1;
+		_dtr(rt, v.value);
+		nr = 1;
 		break;
 
 	case VALUE_CHAIN:
@@ -178,9 +179,9 @@ unsigned radix_tree_size(struct radix_tree *rt)
 	return rt->nr_entries;
 }
 
-static bool _insert(struct radix_tree *rt, struct value *v, uint8_t *kb, uint8_t *ke, union radix_value rv);
+static bool _insert(struct radix_tree *rt, struct value *v, const uint8_t *kb, const uint8_t *ke, union radix_value rv);
 
-static bool _insert_unset(struct radix_tree *rt, struct value *v, uint8_t *kb, uint8_t *ke, union radix_value rv)
+static bool _insert_unset(struct radix_tree *rt, struct value *v, const uint8_t *kb, const uint8_t *ke, union radix_value rv)
 {
 	unsigned len = ke - kb;
 
@@ -190,8 +191,8 @@ static bool _insert_unset(struct radix_tree *rt, struct value *v, uint8_t *kb, u
 		v->value = rv;
 		rt->nr_entries++;
 	} else {
-		// prefix -> value
-		struct prefix_chain *pc = zalloc(sizeof(*pc) + len);
+		// prefix -> value (all fields explicitly initialized)
+		struct prefix_chain *pc = malloc(sizeof(*pc) + len);
 		if (!pc)
 			return false;
 
@@ -207,7 +208,7 @@ static bool _insert_unset(struct radix_tree *rt, struct value *v, uint8_t *kb, u
 	return true;
 }
 
-static bool _insert_value(struct radix_tree *rt, struct value *v, uint8_t *kb, uint8_t *ke, union radix_value rv)
+static bool _insert_value(struct radix_tree *rt, struct value *v, const uint8_t *kb, const uint8_t *ke, union radix_value rv)
 {
 	unsigned len = ke - kb;
 
@@ -234,7 +235,7 @@ static bool _insert_value(struct radix_tree *rt, struct value *v, uint8_t *kb, u
 	return true;
 }
 
-static bool _insert_value_chain(struct radix_tree *rt, struct value *v, uint8_t *kb, uint8_t *ke, union radix_value rv)
+static bool _insert_value_chain(struct radix_tree *rt, struct value *v, const uint8_t *kb, const uint8_t *ke, union radix_value rv)
 {
 	struct value_chain *vc = v->value.ptr;
 	return _insert(rt, &vc->child, kb, ke, rv);
@@ -248,7 +249,7 @@ static unsigned min(unsigned lhs, unsigned rhs)
 		return rhs;
 }
 
-static bool _insert_prefix_chain(struct radix_tree *rt, struct value *v, uint8_t *kb, uint8_t *ke, union radix_value rv)
+static bool _insert_prefix_chain(struct radix_tree *rt, struct value *v, const uint8_t *kb, const uint8_t *ke, union radix_value rv)
 {
 	struct prefix_chain *pc = v->value.ptr;
 
@@ -266,7 +267,8 @@ static bool _insert_prefix_chain(struct radix_tree *rt, struct value *v, uint8_t
 			if (kb[i] != pc->prefix[i])
 				break;
 
-		if (!(pc2 = zalloc(sizeof(*pc2) + pc->len - i)))
+		// All fields of pc2 are explicitly initialized
+		if (!(pc2 = malloc(sizeof(*pc2) + pc->len - i)))
 			return false;
 		pc2->len = pc->len - i;
 		memmove(pc2->prefix, pc->prefix + i, pc2->len);
@@ -278,7 +280,7 @@ static bool _insert_prefix_chain(struct radix_tree *rt, struct value *v, uint8_t
 		pc->len = i;
 
 		if (!_insert(rt, &pc->child, kb + i, ke, rv)) {
-			free(pc2);
+			free(pc->child.value.ptr);
 			return false;
 		}
 
@@ -292,6 +294,7 @@ static bool _insert_prefix_chain(struct radix_tree *rt, struct value *v, uint8_t
 		if (pc->len == 1) {
 			n4->values[0] = pc->child;
 			free(pc);
+			v->value.ptr = NULL;
 		} else {
 			memmove(pc->prefix, pc->prefix + 1, pc->len - 1);
 			pc->len--;
@@ -313,7 +316,7 @@ static bool _insert_prefix_chain(struct radix_tree *rt, struct value *v, uint8_t
 	return true;
 }
 
-static bool _insert_node4(struct radix_tree *rt, struct value *v, uint8_t *kb, uint8_t *ke, union radix_value rv)
+static bool _insert_node4(struct radix_tree *rt, struct value *v, const uint8_t *kb, const uint8_t *ke, union radix_value rv)
 {
 	struct node4 *n4 = v->value.ptr;
 	if (n4->nr_entries == 4) {
@@ -343,7 +346,7 @@ static bool _insert_node4(struct radix_tree *rt, struct value *v, uint8_t *kb, u
 	return true;
 }
 
-static bool _insert_node16(struct radix_tree *rt, struct value *v, uint8_t *kb, uint8_t *ke, union radix_value rv)
+static bool _insert_node16(struct radix_tree *rt, struct value *v, const uint8_t *kb, const uint8_t *ke, union radix_value rv)
 {
 	struct node16 *n16 = v->value.ptr;
 
@@ -382,7 +385,7 @@ static bool _insert_node16(struct radix_tree *rt, struct value *v, uint8_t *kb,
 	return true;
 }
 
-static bool _insert_node48(struct radix_tree *rt, struct value *v, uint8_t *kb, uint8_t *ke, union radix_value rv)
+static bool _insert_node48(struct radix_tree *rt, struct value *v, const uint8_t *kb, const uint8_t *ke, union radix_value rv)
 {
 	struct node48 *n48 = v->value.ptr;
 	if (n48->nr_entries == 48) {
@@ -417,20 +420,20 @@ static bool _insert_node48(struct radix_tree *rt, struct value *v, uint8_t *kb,
 	return true;
 }
 
-static bool _insert_node256(struct radix_tree *rt, struct value *v, uint8_t *kb, uint8_t *ke, union radix_value rv)
+static bool _insert_node256(struct radix_tree *rt, struct value *v, const uint8_t *kb, const uint8_t *ke, union radix_value rv)
 {
 	struct node256 *n256 = v->value.ptr;
 	bool r, was_unset = n256->values[*kb].type == UNSET;
 
 	r = _insert(rt, n256->values + *kb, kb + 1, ke, rv);
 	if (r && was_unset)
-        	n256->nr_entries++;
+		n256->nr_entries++;
 
 	return r;
 }
 
 // FIXME: the tree should not be touched if insert fails (eg, OOM)
-static bool _insert(struct radix_tree *rt, struct value *v, uint8_t *kb, uint8_t *ke, union radix_value rv)
+static bool _insert(struct radix_tree *rt, struct value *v, const uint8_t *kb, const uint8_t *ke, union radix_value rv)
 {
 	if (kb == ke) {
 		if (v->type == UNSET) {
@@ -442,7 +445,8 @@ static bool _insert(struct radix_tree *rt, struct value *v, uint8_t *kb, uint8_t
 			v->value = rv;
 
 		} else {
-			struct value_chain *vc = zalloc(sizeof(*vc));
+			// All fields explicitly initialized
+			struct value_chain *vc = malloc(sizeof(*vc));
 			if (!vc)
 				return false;
 
@@ -487,10 +491,10 @@ static bool _insert(struct radix_tree *rt, struct value *v, uint8_t *kb, uint8_t
 
 struct lookup_result {
 	struct value *v;
-	uint8_t *kb;
+	const uint8_t *kb;
 };
 
-static struct lookup_result _lookup_prefix(struct value *v, uint8_t *kb, uint8_t *ke)
+static struct lookup_result _lookup_prefix(struct value *v, const uint8_t *kb, const uint8_t *ke)
 {
 	unsigned i;
 	struct value_chain *vc;
@@ -500,7 +504,7 @@ static struct lookup_result _lookup_prefix(struct value *v, uint8_t *kb, uint8_t
 	struct node48 *n48;
 	struct node256 *n256;
 
-	if (kb == ke)
+	if (kb == ke || !kb) /* extra check for !kb for coverity */
 		return (struct lookup_result) {.v = v, .kb = kb};
 
 	switch (v->type) {
@@ -555,45 +559,38 @@ static struct lookup_result _lookup_prefix(struct value *v, uint8_t *kb, uint8_t
 	return (struct lookup_result) {.v = v, .kb = kb};
 }
 
-bool radix_tree_insert(struct radix_tree *rt, uint8_t *kb, uint8_t *ke, union radix_value rv)
+bool radix_tree_insert(struct radix_tree *rt, const void *key, size_t keylen, union radix_value rv)
 {
+	const uint8_t *kb = key;
+	const uint8_t *ke = kb + keylen;
 	struct lookup_result lr = _lookup_prefix(&rt->root, kb, ke);
 	return _insert(rt, lr.v, lr.kb, ke, rv);
 }
 
-// Note the degrade functions also free the original node.
-static void _degrade_to_n4(struct node16 *n16, struct value *result)
+int radix_tree_uniq_insert(struct radix_tree *rt, const void *key, size_t keylen, union radix_value rv)
 {
-        struct node4 *n4 = zalloc(sizeof(*n4));
-
-	assert(n4 != NULL);
-
-        n4->nr_entries = n16->nr_entries;
-        memcpy(n4->keys, n16->keys, n16->nr_entries * sizeof(*n4->keys));
-        memcpy(n4->values, n16->values, n16->nr_entries * sizeof(*n4->values));
-        free(n16);
-
-	result->type = NODE4;
-	result->value.ptr = n4;
+	unsigned entries = rt->nr_entries;
+	return radix_tree_insert(rt, key, keylen, rv) ?
+		((entries != rt->nr_entries) ? 1 : -1) : 0;
 }
 
 static void _degrade_to_n16(struct node48 *n48, struct value *result)
 {
 	unsigned i, count = 0;
-        struct node16 *n16 = zalloc(sizeof(*n16));
+	struct node16 *n16 = zalloc(sizeof(*n16));
 
 	assert(n16 != NULL);
 
-        n16->nr_entries = n48->nr_entries;
-        for (i = 0; i < 256; i++) {
-	        if (n48->keys[i] < 48) {
-		        n16->keys[count] = i;
-		        n16->values[count] = n48->values[n48->keys[i]];
-		        count++;
-	        }
-        }
+	n16->nr_entries = n48->nr_entries;
+	for (i = 0; i < 256; i++) {
+		if (n48->keys[i] < 48) {
+			n16->keys[count] = i;
+			n16->values[count] = n48->values[n48->keys[i]];
+			count++;
+		}
+	}
 
-        free(n48);
+	free(n48);
 
 	result->type = NODE16;
 	result->value.ptr = n16;
@@ -601,13 +598,13 @@ static void _degrade_to_n16(struct node48 *n48, struct value *result)
 
 static void _degrade_to_n48(struct node256 *n256, struct value *result)
 {
-        unsigned i, count = 0;
-        struct node48 *n48 = zalloc(sizeof(*n48));
+	unsigned i, count = 0;
+	struct node48 *n48 = zalloc(sizeof(*n48));
 
 	assert(n48 != NULL);
 
-        n48->nr_entries = n256->nr_entries;
-        for (i = 0; i < 256; i++) {
+	n48->nr_entries = n256->nr_entries;
+	for (i = 0; i < 256; i++) {
 		if (n256->values[i].type == UNSET)
 			n48->keys[i] = 48;
 
@@ -616,9 +613,9 @@ static void _degrade_to_n48(struct node256 *n256, struct value *result)
 			n48->values[count] = n256->values[i];
 			count++;
 		}
-        }
+	}
 
-        free(n256);
+	free(n256);
 
 	result->type = NODE48;
 	result->value.ptr = n48;
@@ -632,14 +629,14 @@ static void _erase_elt(void *array, size_t obj_size, unsigned count, unsigned id
 		return;
 
 	memmove(((uint8_t *) array) + (obj_size * idx),
-                ((uint8_t *) array) + (obj_size * (idx + 1)),
-                obj_size * (count - idx - 1));
+		((uint8_t *) array) + (obj_size * (idx + 1)),
+		obj_size * (count - idx - 1));
 
 	// Zero the now unused last elt (set's v.type to UNSET)
 	memset(((uint8_t *) array) + (count - 1) * obj_size, 0, obj_size);
 }
 
-static bool _remove(struct radix_tree *rt, struct value *root, uint8_t *kb, uint8_t *ke)
+static bool _remove(struct radix_tree *rt, struct value *root, const uint8_t *kb, const uint8_t *ke)
 {
 	bool r;
 	unsigned i, j;
@@ -651,27 +648,28 @@ static bool _remove(struct radix_tree *rt, struct value *root, uint8_t *kb, uint
 	struct node256 *n256;
 
 	if (kb == ke) {
-        	if (root->type == VALUE) {
-                	root->type = UNSET;
-                	_dtr(rt, root->value);
-                	return true;
+		if (root->type == VALUE) {
+			root->type = UNSET;
+			_dtr(rt, root->value);
+			return true;
 
-                } else if (root->type == VALUE_CHAIN) {
+		} else if (root->type == VALUE_CHAIN) {
+			// Free value_chain after copying child out
 			vc = root->value.ptr;
 			_dtr(rt, vc->value);
-			memcpy(root, &vc->child, sizeof(*root));
+			*root = vc->child;
 			free(vc);
 			return true;
 
-                } else
+		} else
 			return false;
 	}
 
 	switch (root->type) {
 	case UNSET:
 	case VALUE:
-        	// this is a value for a prefix of the key
-        	return false;
+		// this is a value for a prefix of the key
+		return false;
 
 	case VALUE_CHAIN:
 		vc = root->value.ptr;
@@ -686,11 +684,11 @@ static bool _remove(struct radix_tree *rt, struct value *root, uint8_t *kb, uint
 	case PREFIX_CHAIN:
 		pc = root->value.ptr;
 		if (ke - kb < pc->len)
-        		return false;
+			return false;
 
 		for (i = 0; i < pc->len; i++)
 			if (kb[i] != pc->prefix[i])
-        			return false;
+				return false;
 
 		r = _remove(rt, &pc->child, kb + pc->len, ke);
 		if (r && pc->child.type == UNSET) {
@@ -705,12 +703,12 @@ static bool _remove(struct radix_tree *rt, struct value *root, uint8_t *kb, uint
 			if (n4->keys[i] == *kb) {
 				r = _remove(rt, n4->values + i, kb + 1, ke);
 				if (r && n4->values[i].type == UNSET) {
-        				if (i < n4->nr_entries) {
-	        				_erase_elt(n4->keys, sizeof(*n4->keys), n4->nr_entries, i);
-	        				_erase_elt(n4->values, sizeof(*n4->values), n4->nr_entries, i);
-        				}
+					if (i < n4->nr_entries) {
+						_erase_elt(n4->keys, sizeof(*n4->keys), n4->nr_entries, i);
+						_erase_elt(n4->values, sizeof(*n4->values), n4->nr_entries, i);
+					}
 
-        				n4->nr_entries--;
+					n4->nr_entries--;
 					if (!n4->nr_entries) {
 						free(n4);
 						root->type = UNSET;
@@ -722,19 +720,20 @@ static bool _remove(struct radix_tree *rt, struct value *root, uint8_t *kb, uint
 		return false;
 
 	case NODE16:
-        	n16 = root->value.ptr;
+		n16 = root->value.ptr;
 		for (i = 0; i < n16->nr_entries; i++) {
 			if (n16->keys[i] == *kb) {
 				r = _remove(rt, n16->values + i, kb + 1, ke);
 				if (r && n16->values[i].type == UNSET) {
-        				if (i < n16->nr_entries) {
-	        				_erase_elt(n16->keys, sizeof(*n16->keys), n16->nr_entries, i);
-	        				_erase_elt(n16->values, sizeof(*n16->values), n16->nr_entries, i);
-        				}
+					if (i < n16->nr_entries) {
+						_erase_elt(n16->keys, sizeof(*n16->keys), n16->nr_entries, i);
+						_erase_elt(n16->values, sizeof(*n16->values), n16->nr_entries, i);
+					}
 
-        				n16->nr_entries--;
-					if (n16->nr_entries <= 4) {
-        					_degrade_to_n4(n16, root);
+					n16->nr_entries--;
+					if (!n16->nr_entries) {
+						free(n16);
+						root->type = UNSET;
 					}
 				}
 				return r;
@@ -746,18 +745,18 @@ static bool _remove(struct radix_tree *rt, struct value *root, uint8_t *kb, uint
 		n48 = root->value.ptr;
 		i = n48->keys[*kb];
 		if (i < 48) {
-        		r = _remove(rt, n48->values + i, kb + 1, ke);
-        		if (r && n48->values[i].type == UNSET) {
-                		n48->keys[*kb] = 48;
-                		for (j = 0; j < 256; j++)
-	                		if (n48->keys[j] < 48 && n48->keys[j] > i)
-		                		n48->keys[j]--;
+			r = _remove(rt, n48->values + i, kb + 1, ke);
+			if (r && n48->values[i].type == UNSET) {
+				n48->keys[*kb] = 48;
+				for (j = 0; j < 256; j++)
+					if (n48->keys[j] < 48 && n48->keys[j] > i)
+						n48->keys[j]--;
 				_erase_elt(n48->values, sizeof(*n48->values), n48->nr_entries, i);
 				n48->nr_entries--;
 				if (n48->nr_entries <= 16)
-        				_degrade_to_n16(n48, root);
-        		}
-        		return r;
+					_degrade_to_n16(n48, root);
+			}
+			return r;
 		}
 		return false;
 
@@ -767,7 +766,7 @@ static bool _remove(struct radix_tree *rt, struct value *root, uint8_t *kb, uint
 		if (r && n256->values[*kb].type == UNSET) {
 			n256->nr_entries--;
 			if (n256->nr_entries <= 48)
-        			_degrade_to_n48(n256, root);
+				_degrade_to_n48(n256, root);
 		}
 		return r;
 	}
@@ -775,11 +774,14 @@ static bool _remove(struct radix_tree *rt, struct value *root, uint8_t *kb, uint
 	return false;
 }
 
-bool radix_tree_remove(struct radix_tree *rt, uint8_t *key_begin, uint8_t *key_end)
+bool radix_tree_remove(struct radix_tree *rt, const void *key, size_t keylen)
 {
-	if (_remove(rt, &rt->root, key_begin, key_end)) {
-        	rt->nr_entries--;
-        	return true;
+	const uint8_t *kb = key;
+	const uint8_t *ke = kb + keylen;
+
+	if (_remove(rt, &rt->root, kb, ke)) {
+		rt->nr_entries--;
+		return true;
 	}
 
 	return false;
@@ -787,25 +789,25 @@ bool radix_tree_remove(struct radix_tree *rt, uint8_t *key_begin, uint8_t *key_e
 
 //----------------------------------------------------------------
 
-static bool _prefix_chain_matches(struct lookup_result *lr, uint8_t *ke)
+static bool _prefix_chain_matches(const struct lookup_result *lr, const uint8_t *ke)
 {
-        // It's possible the top node is a prefix chain, and
-        // the remaining key matches part of it.
-        if (lr->v->type == PREFIX_CHAIN) {
-                unsigned i, rlen = ke - lr->kb;
-                struct prefix_chain *pc = lr->v->value.ptr;
-                if (rlen < pc->len) {
-                        for (i = 0; i < rlen; i++)
-                                if (pc->prefix[i] != lr->kb[i])
-                                        return false;
-                        return true;
+	// It's possible the top node is a prefix chain, and
+	// the remaining key matches part of it.
+	if (lr->v->type == PREFIX_CHAIN) {
+		unsigned i, rlen = ke - lr->kb;
+		const struct prefix_chain *pc = lr->v->value.ptr;
+		if (rlen < pc->len) {
+			for (i = 0; i < rlen; i++)
+				if (pc->prefix[i] != lr->kb[i])
+					return false;
+			return true;
 		}
-        }
+	}
 
-        return false;
+	return false;
 }
 
-static bool _remove_subtree(struct radix_tree *rt, struct value *root, uint8_t *kb, uint8_t *ke, unsigned *count)
+static bool _remove_subtree(struct radix_tree *rt, struct value *root, const uint8_t *kb, const uint8_t *ke, unsigned *count)
 {
 	bool r;
 	unsigned i, j, len;
@@ -826,7 +828,7 @@ static bool _remove_subtree(struct radix_tree *rt, struct value *root, uint8_t *
 	case UNSET:
 	case VALUE:
 		// No entries with the given prefix
-        	return true;
+		return true;
 
 	case VALUE_CHAIN:
 		vc = root->value.ptr;
@@ -843,7 +845,7 @@ static bool _remove_subtree(struct radix_tree *rt, struct value *root, uint8_t *
 		len = min(pc->len, ke - kb);
 		for (i = 0; i < len; i++)
 			if (kb[i] != pc->prefix[i])
-        			return true;
+				return true;
 
 		r = _remove_subtree(rt, &pc->child, len < pc->len ? ke : (kb + pc->len), ke, count);
 		if (r && pc->child.type == UNSET) {
@@ -858,12 +860,12 @@ static bool _remove_subtree(struct radix_tree *rt, struct value *root, uint8_t *
 			if (n4->keys[i] == *kb) {
 				r = _remove_subtree(rt, n4->values + i, kb + 1, ke, count);
 				if (r && n4->values[i].type == UNSET) {
-        				if (i < n4->nr_entries) {
-	        				_erase_elt(n4->keys, sizeof(*n4->keys), n4->nr_entries, i);
-	        				_erase_elt(n4->values, sizeof(*n4->values), n4->nr_entries, i);
-        				}
+					if (i < n4->nr_entries) {
+						_erase_elt(n4->keys, sizeof(*n4->keys), n4->nr_entries, i);
+						_erase_elt(n4->values, sizeof(*n4->values), n4->nr_entries, i);
+					}
 
-        				n4->nr_entries--;
+					n4->nr_entries--;
 					if (!n4->nr_entries) {
 						free(n4);
 						root->type = UNSET;
@@ -875,19 +877,21 @@ static bool _remove_subtree(struct radix_tree *rt, struct value *root, uint8_t *
 		return true;
 
 	case NODE16:
-        	n16 = root->value.ptr;
+		n16 = root->value.ptr;
 		for (i = 0; i < n16->nr_entries; i++) {
 			if (n16->keys[i] == *kb) {
 				r = _remove_subtree(rt, n16->values + i, kb + 1, ke, count);
 				if (r && n16->values[i].type == UNSET) {
-        				if (i < n16->nr_entries) {
-	        				_erase_elt(n16->keys, sizeof(*n16->keys), n16->nr_entries, i);
-	        				_erase_elt(n16->values, sizeof(*n16->values), n16->nr_entries, i);
-        				}
+					if (i < n16->nr_entries) {
+						_erase_elt(n16->keys, sizeof(*n16->keys), n16->nr_entries, i);
+						_erase_elt(n16->values, sizeof(*n16->values), n16->nr_entries, i);
+					}
 
-        				n16->nr_entries--;
-					if (n16->nr_entries <= 4)
-        					_degrade_to_n4(n16, root);
+					n16->nr_entries--;
+					if (!n16->nr_entries) {
+						free(n16);
+						root->type = UNSET;
+					}
 				}
 				return r;
 			}
@@ -898,18 +902,18 @@ static bool _remove_subtree(struct radix_tree *rt, struct value *root, uint8_t *
 		n48 = root->value.ptr;
 		i = n48->keys[*kb];
 		if (i < 48) {
-        		r = _remove_subtree(rt, n48->values + i, kb + 1, ke, count);
-        		if (r && n48->values[i].type == UNSET) {
-                		n48->keys[*kb] = 48;
-                		for (j = 0; j < 256; j++)
-	                		if (n48->keys[j] < 48 && n48->keys[j] > i)
-		                		n48->keys[j]--;
+			r = _remove_subtree(rt, n48->values + i, kb + 1, ke, count);
+			if (r && n48->values[i].type == UNSET) {
+				n48->keys[*kb] = 48;
+				for (j = 0; j < 256; j++)
+					if (n48->keys[j] < 48 && n48->keys[j] > i)
+						n48->keys[j]--;
 				_erase_elt(n48->values, sizeof(*n48->values), n48->nr_entries, i);
 				n48->nr_entries--;
 				if (n48->nr_entries <= 16)
-        				_degrade_to_n16(n48, root);
-        		}
-        		return r;
+					_degrade_to_n16(n48, root);
+			}
+			return r;
 		}
 		return true;
 
@@ -922,7 +926,7 @@ static bool _remove_subtree(struct radix_tree *rt, struct value *root, uint8_t *
 		if (r && n256->values[*kb].type == UNSET) {
 			n256->nr_entries--;
 			if (n256->nr_entries <= 48)
-        			_degrade_to_n48(n256, root);
+				_degrade_to_n48(n256, root);
 		}
 		return r;
 	}
@@ -931,11 +935,13 @@ static bool _remove_subtree(struct radix_tree *rt, struct value *root, uint8_t *
 	return false;
 }
 
-unsigned radix_tree_remove_prefix(struct radix_tree *rt, uint8_t *kb, uint8_t *ke)
+unsigned radix_tree_remove_prefix(struct radix_tree *rt, const void *prefix, size_t prefix_len)
 {
-        unsigned count = 0;
+	const uint8_t *kb = prefix;
+	const uint8_t *ke = kb + prefix_len;
+	unsigned count = 0;
 
-        if (_remove_subtree(rt, &rt->root, kb, ke, &count))
+	if (_remove_subtree(rt, &rt->root, kb, ke, &count))
 		rt->nr_entries -= count;
 
 	return count;
@@ -943,9 +949,11 @@ unsigned radix_tree_remove_prefix(struct radix_tree *rt, uint8_t *kb, uint8_t *k
 
 //----------------------------------------------------------------
 
-bool radix_tree_lookup(struct radix_tree *rt,
-		       uint8_t *kb, uint8_t *ke, union radix_value *result)
+bool radix_tree_lookup(struct radix_tree *rt, const void *key, size_t keylen,
+		       union radix_value *result)
 {
+	const uint8_t *kb = key;
+	const uint8_t *ke = kb + keylen;
 	struct value_chain *vc;
 	struct lookup_result lr = _lookup_prefix(&rt->root, kb, ke);
 	if (lr.kb == ke) {
@@ -968,58 +976,58 @@ bool radix_tree_lookup(struct radix_tree *rt,
 }
 
 // FIXME: build up the keys too
-static bool _iterate(struct value *v, struct radix_tree_iterator *it)
+static bool _iterate(struct radix_tree_iterator *it, const struct value *v)
 {
 	unsigned i;
-	struct value_chain *vc;
-	struct prefix_chain *pc;
-	struct node4 *n4;
-	struct node16 *n16;
-	struct node48 *n48;
-	struct node256 *n256;
+	const struct value_chain *vc;
+	const struct prefix_chain *pc;
+	const struct node4 *n4;
+	const struct node16 *n16;
+	const struct node48 *n48;
+	const struct node256 *n256;
 
 	switch (v->type) {
 	case UNSET:
-        	// can't happen
+		// can't happen
 		break;
 
 	case VALUE:
-        	return it->visit(it, NULL, NULL, v->value);
+		return it->visit(it, NULL, 0, v->value);
 
 	case VALUE_CHAIN:
 		vc = v->value.ptr;
-		return it->visit(it, NULL, NULL, vc->value) && _iterate(&vc->child, it);
+		return it->visit(it, NULL, 0, vc->value) && _iterate(it, &vc->child);
 
 	case PREFIX_CHAIN:
 		pc = v->value.ptr;
-		return _iterate(&pc->child, it);
+		return _iterate(it, &pc->child);
 
 	case NODE4:
-		n4 = (struct node4 *) v->value.ptr;
+		n4 = (const struct node4 *) v->value.ptr;
 		for (i = 0; i < n4->nr_entries; i++)
-			if (!_iterate(n4->values + i, it))
-        			return false;
-        	return true;
+			if (!_iterate(it, n4->values + i))
+				return false;
+		return true;
 
 	case NODE16:
-		n16 = (struct node16 *) v->value.ptr;
+		n16 = (const struct node16 *) v->value.ptr;
 		for (i = 0; i < n16->nr_entries; i++)
-        		if (!_iterate(n16->values + i, it))
-        			return false;
+			if (!_iterate(it, n16->values + i))
+				return false;
 		return true;
 
 	case NODE48:
-		n48 = (struct node48 *) v->value.ptr;
+		n48 = (const struct node48 *) v->value.ptr;
 		for (i = 0; i < n48->nr_entries; i++)
-        		if (!_iterate(n48->values + i, it))
-        			return false;
+			if (!_iterate(it, n48->values + i))
+				return false;
 		return true;
 
 	case NODE256:
-		n256 = (struct node256 *) v->value.ptr;
+		n256 = (const struct node256 *) v->value.ptr;
 		for (i = 0; i < 256; i++)
-        		if (n256->values[i].type != UNSET && !_iterate(n256->values + i, it))
-        			return false;
+			if (n256->values[i].type != UNSET && !_iterate(it, n256->values + i))
+				return false;
 		return true;
 	}
 
@@ -1027,12 +1035,14 @@ static bool _iterate(struct value *v, struct radix_tree_iterator *it)
 	return false;
 }
 
-void radix_tree_iterate(struct radix_tree *rt, uint8_t *kb, uint8_t *ke,
-                        struct radix_tree_iterator *it)
+void radix_tree_iterate(struct radix_tree *rt, const void *key, size_t keylen,
+			struct radix_tree_iterator *it)
 {
+	const uint8_t *kb = key;
+	const uint8_t *ke = kb + keylen;
 	struct lookup_result lr = _lookup_prefix(&rt->root, kb, ke);
 	if (lr.kb == ke || _prefix_chain_matches(&lr, ke))
-		(void) _iterate(lr.v, it);
+		(void) _iterate(it, lr.v);
 }
 
 //----------------------------------------------------------------
@@ -1130,7 +1140,7 @@ static bool _check_nodes(struct value *v, unsigned *count)
 
 		if (ncount != n48->nr_entries) {
 			fprintf(stderr, "incorrect number of entries in n48, n48->nr_entries = %u, actual = %u\n",
-                                n48->nr_entries, ncount);
+				n48->nr_entries, ncount);
 			return false;
 		}
 
@@ -1166,7 +1176,7 @@ static bool _check_nodes(struct value *v, unsigned *count)
 
 		if (ncount != n256->nr_entries) {
 			fprintf(stderr, "incorrect number of entries in n256, n256->nr_entries = %u, actual = %u\n",
-                                n256->nr_entries, ncount);
+				n256->nr_entries, ncount);
 			return false;
 		}
 
@@ -1189,7 +1199,7 @@ bool radix_tree_is_well_formed(struct radix_tree *rt)
 
 	if (rt->nr_entries != count) {
 		fprintf(stderr, "incorrect entry count: rt->nr_entries = %u, actual = %u\n",
-                        rt->nr_entries, count);
+			rt->nr_entries, count);
 		return false;
 	}
 
@@ -1207,6 +1217,7 @@ static void _dump(FILE *out, struct value v, unsigned indent)
 	struct node16 *n16;
 	struct node48 *n48;
 	struct node256 *n256;
+	unsigned printable;
 
 	if (v.type == UNSET)
 		return;
@@ -1231,9 +1242,22 @@ static void _dump(FILE *out, struct value v, unsigned indent)
 
 	case PREFIX_CHAIN:
 		pc = v.value.ptr;
-		fprintf(out, "<prefix: ");
+		fprintf(out, "<prefix(%u): ", pc->len);
+		printable = 1;
 		for (i = 0; i < pc->len; i++)
-			fprintf(out, "%x.", (unsigned) *(pc->prefix + i));
+			if (!isprint(pc->prefix[i])) {
+				printable = 0;
+				break;
+			}
+		if (printable)
+			fputc('"', out);
+		for (i = 0; i < pc->len; i++)
+			if (printable)
+				fprintf(out, "%c", pc->prefix[i]);
+			else
+				fprintf(out, "%02x.", (unsigned) *(pc->prefix + i));
+		if (printable)
+			fputc('"', out);
 		fprintf(out, ">\n");
 		_dump(out, pc->child, indent + 1);
 		break;
@@ -1242,7 +1266,7 @@ static void _dump(FILE *out, struct value v, unsigned indent)
 		n4 = v.value.ptr;
 		fprintf(out, "<n4: ");
 		for (i = 0; i < n4->nr_entries; i++)
-			fprintf(out, "%x ", (unsigned) n4->keys[i]);
+			fprintf(out, "%02x ", (unsigned) n4->keys[i]);
 		fprintf(out, ">\n");
 
 		for (i = 0; i < n4->nr_entries; i++)
@@ -1253,7 +1277,7 @@ static void _dump(FILE *out, struct value v, unsigned indent)
 		n16 = v.value.ptr;
 		fprintf(out, "<n16: ");
 		for (i = 0; i < n16->nr_entries; i++)
-			fprintf(out, "%x ", (unsigned) n16->keys[i]);
+			fprintf(out, "%02x ", (unsigned) n16->keys[i]);
 		fprintf(out, ">\n");
 
 		for (i = 0; i < n16->nr_entries; i++)
@@ -1265,7 +1289,7 @@ static void _dump(FILE *out, struct value v, unsigned indent)
 		fprintf(out, "<n48: ");
 		for (i = 0; i < 256; i++)
 			if (n48->keys[i] < 48)
-				fprintf(out, "%x ", i);
+				fprintf(out, "%02x ", i);
 		fprintf(out, ">\n");
 
 		for (i = 0; i < n48->nr_entries; i++) {
@@ -1279,7 +1303,7 @@ static void _dump(FILE *out, struct value v, unsigned indent)
 		fprintf(out, "<n256: ");
 		for (i = 0; i < 256; i++)
 			if (n256->values[i].type != UNSET)
-				fprintf(out, "%x ", i);
+				fprintf(out, "%02x ", i);
 		fprintf(out, ">\n");
 
 		for (i = 0; i < 256; i++)
