@@ -16,9 +16,12 @@
 #define LVMLOCKD_SOCKET DEFAULT_RUN_DIR "/lvmlockd.socket"
 #define LVMLOCKD_ADOPT_FILE DEFAULT_RUN_DIR "/lvmlockd.adopt"
 
+#define LVMLOCKD_USE_SANLOCK_LVB 0
+
 /* Wrappers to open/close connection */
 
-static inline daemon_handle lvmlockd_open(const char *sock)
+static inline __attribute__((always_inline))
+	daemon_handle lvmlockd_open(const char *sock)
 {
 	daemon_info lvmlockd_info = {
 		.path = "lvmlockd",
@@ -51,5 +54,17 @@ static inline void lvmlockd_close(daemon_handle h)
 #define EREMOVED  219
 #define EDEVOPEN  220 /* sanlock failed to open lvmlock LV */
 #define ELMERR    221
+#define EORPHAN   222
+#define EADOPT_NONE  223
+#define EADOPT_RETRY 224
+#define EIOTIMEOUT   225
+#define ELOCKREPAIR  226
+
+#define LOCKARGS_VERSION	0x00000001 /* meta only */
+#define LOCKARGS_LVMLOCK	0x00000002 /* meta only */
+#define LOCKARGS_TIMEOUT        0x00000004 /* user only */
+#define LOCKARGS_NOTIMEOUT      0x00000008 /* meta or user */
+#define LOCKARGS_PERSIST        0x00000010 /* meta or user */
+#define LOCKARGS_NOPERSIST      0x00000020 /* user only */
 
 #endif	/* _LVM_LVMLOCKD_CLIENT_H */
