@@ -19,10 +19,12 @@
 #include "lib/display/display.h"
 #include "lib/misc/lvm-exec.h"
 #include "lib/activate/dev_manager.h"
+#include "lib/commands/toolcontext.h"
 
 #include <dirent.h>
 #include <mntent.h>
 #include <sys/ioctl.h>
+#include <linux/types.h>
 
 static const char *_get_lvresize_fs_helper_path(struct cmd_context *cmd)
 {
@@ -181,6 +183,8 @@ static int _btrfs_get_mnt(struct fs_info *fsi, dev_t lv_devt)
 			fsi->mounted = 0;
 			return 1;
 		}
+		log_sys_error("opendir", devices_path);
+		return 0;
 	}
 
 	/*
@@ -251,8 +255,8 @@ static int _btrfs_get_mnt(struct fs_info *fsi, dev_t lv_devt)
 	return ret;
 }
 
-int fs_get_info(struct cmd_context *cmd, struct logical_volume *lv,
-		struct fs_info *fsi, int include_mount)
+
+int fs_get_info(struct cmd_context *cmd, struct logical_volume *lv, struct fs_info *fsi)
 {
 	char lv_path[PATH_MAX];
 	char crypt_path[PATH_MAX] = { 0 };
@@ -338,14 +342,17 @@ int fs_get_info(struct cmd_context *cmd, struct logical_volume *lv,
 		st_top = st_lv;
 	}
 
-	if (!include_mount)
-		return 1;
-
 	if (!strcmp(fsi->fstype, "btrfs"))
 		ret = _btrfs_get_mnt(fsi, st_lv.st_rdev);
 	else
 		ret = _fs_get_mnt(fsi, st_top.st_rdev);
 
+	/* blkid FSLASTBLOCK may be incorrect for mounted xfs */
+	if (fsi->mounted && !strcmp(fsi->fstype, "xfs")) {
+		if (!(ret = fs_xfs_update_size_mounted(cmd, lv, lv_path, fsi)))
+			stack;
+	}
+
 	fsi->unmounted = !fsi->mounted;
 	return ret;
 }
@@ -359,6 +366,8 @@ int fs_mount_state_is_misnamed(struct cmd_context *cmd, struct logical_volume *l
 	char proc_mntpath[PATH_MAX + 1];
 	char mtab_mntpath[PATH_MAX] = { 0 };
 	char dm_devpath[PATH_MAX];
+	char dm_devpath_resolved[PATH_MAX];
+	char proc_devpath_resolved[PATH_MAX];
 	char tmp_path[PATH_MAX];
 	char *dm_name;
 	struct stat st_lv;
@@ -458,9 +467,19 @@ int fs_mount_state_is_misnamed(struct cmd_context *cmd, struct logical_volume *l
 		 * it appears in /proc/mounts once as
 		 * /dev/mapper/vg-lvol0 on /foo type xfs ...
 		 */
-
 		dir_match = !strcmp(mtab_mntpath, proc_mntpath);
-		dev_match = !strcmp(dm_devpath, proc_devpath);
+
+		/*
+		 * Resolve symlinks before comparing device paths. In test environments,
+		 * dm_devpath may be a symlink (e.g., /dev/shm/LVMTEST/dev/mapper/vg-lv
+		 * -> /dev/mapper/vg-lv), while proc_devpath is the resolved real path.
+		 * Compare resolved paths to avoid false positives for rename detection.
+		 */
+		if (realpath(dm_devpath, dm_devpath_resolved) &&
+		    realpath(proc_devpath, proc_devpath_resolved))
+			dev_match = !strcmp(dm_devpath_resolved, proc_devpath_resolved);
+		else
+			dev_match = !strcmp(dm_devpath, proc_devpath);
 
 		if (!dir_match && !dev_match)
 			continue;
