@@ -200,7 +200,7 @@ int update_cache_pool_params(struct cmd_context *cmd,
 			     uint32_t pool_data_extents,
 			     uint32_t *pool_metadata_extents,
 			     struct logical_volume *metadata_lv,
-			     int *chunk_size_calc_method, uint32_t *chunk_size)
+			     unsigned *chunk_size_calc_policy, uint32_t *chunk_size)
 {
 	uint64_t min_meta_size;
 	uint64_t pool_metadata_size = (uint64_t) *pool_metadata_extents * extent_size;
@@ -212,6 +212,8 @@ int update_cache_pool_params(struct cmd_context *cmd,
 				    DM_CACHE_MIN_DATA_BLOCK_SIZE - 1) /
 				   DM_CACHE_MIN_DATA_BLOCK_SIZE) * DM_CACHE_MIN_DATA_BLOCK_SIZE;
 
+	*chunk_size_calc_policy = CHUNK_SIZE_CALC_POLICY_UNSELECTED;
+
 	if (!*chunk_size) {
 		if (!(*chunk_size = find_config_tree_int(cmd, allocation_cache_pool_chunk_size_CFG,
 							 profile) * 2)) {
@@ -463,7 +465,7 @@ int lv_cache_wait_for_clean(struct logical_volume *cache_lv, int *is_clean)
 
 	//FIXME: use polling to do this...
 	for (;;) {
-		if (cleaner_policy && interruptible_usleep(500000)) {
+		if (cleaner_policy && !sigint_usleep(500000)) {
 			log_error("Flushing of %s aborted.", display_lvname(cache_lv));
 			if (cache_seg->cleaner_policy) {
 				cache_seg->cleaner_policy = 0;
@@ -491,8 +493,10 @@ int lv_cache_wait_for_clean(struct logical_volume *cache_lv, int *is_clean)
 		dm_pool_destroy(status->mem);
 
 		/* Only clear when policy is Clear or mode != writeback */
-		if (!dirty_blocks && (cleaner_policy || !writeback))
+		if (!dirty_blocks && (cleaner_policy || !writeback)) {
+			log_debug("Flush complete.");
 			break;
+		}
 
 		log_print_unless_silent("Flushing " FMTu64 " blocks for cache %s.",
 					dirty_blocks, display_lvname(cache_lv));
@@ -512,11 +516,7 @@ int lv_cache_wait_for_clean(struct logical_volume *cache_lv, int *is_clean)
 		if (!lv_update_and_reload_origin(cache_lv))
 			return_0;
 
-		if (!sync_local_dev_names(cache_lv->vg->cmd)) {
-			log_error("Failed to sync local devices when clearing cache volume %s.",
-				  display_lvname(cache_lv));
-			return 0;
-		}
+		sync_local_dev_names(cache_lv->vg->cmd);
 	}
 
 	/*
@@ -527,11 +527,7 @@ int lv_cache_wait_for_clean(struct logical_volume *cache_lv, int *is_clean)
 		if (!lv_refresh_suspend_resume(lock_lv))
 			return_0;
 
-		if (!sync_local_dev_names(cache_lv->vg->cmd)) {
-			log_error("Failed to sync local devices after final clearing of cache %s.",
-				  display_lvname(cache_lv));
-			return 0;
-		}
+		sync_local_dev_names(cache_lv->vg->cmd);
 	}
 
 	cache_seg->cleaner_policy = 0;
@@ -1041,7 +1037,7 @@ int cache_vol_set_params(struct cmd_context *cmd,
 	 */
 
 	if (!(extent_size = pool_lv->vg->extent_size)) {
-		log_error(INTERNAL_ERROR "Extend size can't be 0.");
+		log_error(INTERNAL_ERROR "Extent size can't be 0.");
 		return 0;
 	}
 	min_meta_size = extent_size;
@@ -1238,7 +1234,7 @@ int cache_set_params(struct lv_segment *seg,
 		    /* TODO: some calc_policy solution for cache ? */
 		    !recalculate_pool_chunk_size_with_dev_hints(pool_seg->lv,
 								seg_lv(pool_seg, 0),
-								THIN_CHUNK_SIZE_CALC_METHOD_GENERIC))
+								CHUNK_SIZE_CALC_POLICY_GENERIC))
 			return_0;
 	}
 
@@ -1256,7 +1252,6 @@ int cache_set_params(struct lv_segment *seg,
  */
 int wipe_cache_pool(struct logical_volume *cache_pool_lv)
 {
-	int r;
 	struct logical_volume *cache_data_lv;
 
 	/* Only unused cache-pool could be activated and wiped */
@@ -1276,25 +1271,11 @@ int wipe_cache_pool(struct logical_volume *cache_pool_lv)
 		return 1;
 	}
 
-	cache_pool_lv->status |= LV_TEMPORARY;
-	if (!activate_lv(cache_pool_lv->vg->cmd, cache_pool_lv)) {
-		log_error("Aborting. Failed to activate cache pool %s.",
+	if (!activate_and_wipe_lv(cache_pool_lv, WIPE_MODE_DO_ZERO, 0, 0)) {
+		log_error("Aborting. Failed to wipe cache pool %s.",
 			  display_lvname(cache_pool_lv));
 		return 0;
 	}
-	cache_pool_lv->status &= ~LV_TEMPORARY;
-	if (!(r = wipe_lv(cache_pool_lv, (struct wipe_params) { .do_zero = 1 }))) {
-		log_error("Aborting. Failed to wipe cache pool %s.",
-			  display_lvname(cache_pool_lv));
-		/* Delay return of error after deactivation */
-	}
 
-	/* Deactivate cleared cache-pool metadata */
-	if (!deactivate_lv(cache_pool_lv->vg->cmd, cache_pool_lv)) {
-		log_error("Aborting. Could not deactivate cache pool %s.",
-			  display_lvname(cache_pool_lv));
-		r = 0;
-	}
-
-	return r;
+	return 1;
 }
