@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2001-2004 Sistina Software, Inc. All rights reserved.
- * Copyright (C) 2004-2012 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2004-2025 Red Hat, Inc. All rights reserved.
  *
  * This file is part of the device-mapper userspace tools.
  *
@@ -14,7 +14,7 @@
  */
 
 #include "libdm/misc/dmlib.h"
-#include "libdm-targets.h"
+#include "libdm/ioctl/libdm-targets.h"
 #include "libdm-common.h"
 #include "libdm/misc/kdev_t.h"
 #include "libdm/misc/dm-ioctl.h"
@@ -336,6 +336,7 @@ struct dm_task *dm_task_create(int type)
 	dmt->new_uuid = 0;
 	dmt->secure_data = 0;
 	dmt->record_timestamp = 0;
+	dmt->ima_measurement = 0;
 
 	return dmt;
 }
@@ -508,7 +509,7 @@ int unmangle_string(const char *str, const char *str_name, size_t len,
 		    char *buf, size_t buf_len, dm_string_mangling_t mode)
 {
 	int strict = mode != DM_STRING_MANGLING_NONE;
-	char str_rest[DM_NAME_LEN];
+	char str_rest[DM_NAME_LEN + 1];
 	size_t i, j;
 	unsigned int code;
 	int r = 0;
@@ -534,7 +535,8 @@ int unmangle_string(const char *str, const char *str_name, size_t len,
 		}
 
 		if (str[i] == '\\' && str[i+1] == 'x') {
-			if (!sscanf(&str[i+2], "%2x%s", &code, str_rest)) {
+			if (!sscanf(&str[i+2], "%2x%" DM_TO_STRING(DM_NAME_LEN) "s",
+				    &code, str_rest)) {
 				log_debug_activation("Hex encoding mismatch detected in %s \"%s\" "
 						     "while trying to unmangle it.", str_name, str);
 				goto out;
@@ -664,7 +666,7 @@ static int _dm_task_set_name_from_path(struct dm_task *dmt, const char *path,
 
 int dm_task_set_name(struct dm_task *dmt, const char *name)
 {
-	char *pos;
+	const char *pos;
 
 	/* Path supplied for existing device? */
 	if ((pos = strrchr(name, '/')))
@@ -1040,12 +1042,19 @@ static int _add_dev_node(const char *dev_name, uint32_t major, uint32_t minor,
 {
 	char path[PATH_MAX];
 	struct stat info;
+	struct stat linfo;
 	dev_t dev = MKDEV(major, minor);
 	mode_t old_mask;
 
 	if (!_build_dev_path(path, sizeof(path), dev_name))
 		return_0;
 
+	/*
+	 * Check if the device node already exists.
+	 * Note: stat() follows symlinks, so this checks the target device,
+	 * not the symlink itself. This works correctly for both real nodes
+	 * and symlinks pointing to the right device.
+	 */
 	if (stat(path, &info) >= 0) {
 		if (!S_ISBLK(info.st_mode)) {
 			log_error("A non-block device file at '%s' "
@@ -1054,17 +1063,71 @@ static int _add_dev_node(const char *dev_name, uint32_t major, uint32_t minor,
 		}
 
 		/* If right inode already exists we don't touch uid etc. */
-		if (info.st_rdev == dev)
+		if (info.st_rdev == dev) {
+			/*
+			 * Correct device exists (either as real node or symlink).
+			 * Use lstat() to distinguish between them for logging.
+			 */
+			if (lstat(path, &linfo) >= 0 && S_ISLNK(linfo.st_mode))
+				log_debug_activation("Symlink %s to correct device already exists", path);
 			return 1;
+		}
 
-		if (unlink(path) < 0) {
-			log_error("Unable to unlink device node for '%s'",
-				  dev_name);
+		if (unlink(path) && (errno != ENOENT)) {
+			log_sys_error("unlink", path);
 			return 0;
 		}
-	} else if (_warn_if_op_needed(warn_if_udev_failed))
-		log_warn("%s not set up by udev: Falling back to direct "
-			 "node creation.", path);
+	} else {
+		/*
+		 * stat() failed. Check for dangling symlinks (where lstat succeeds
+		 * but stat fails with ENOENT because the symlink target doesn't exist).
+		 * Remove the dangling symlink before attempting to create a new node.
+		 */
+		if (errno == ENOENT && lstat(path, &linfo) >= 0 && S_ISLNK(linfo.st_mode)) {
+			log_debug_activation("Removing dangling symlink %s", path);
+			if (unlink(path) && (errno != ENOENT)) {
+				log_sys_error("unlink", path);
+				return 0;
+			}
+		}
+		if (_warn_if_op_needed(warn_if_udev_failed))
+			log_warn("%s not set up by udev: Falling back to direct "
+				 "node creation.", path);
+	}
+
+	/*
+	 * Test environment optimization: If using alternative dev dir (e.g., /tmp/LVMTEST/dev)
+	 * and the real /dev node already exists, create a symlink instead of a duplicate node.
+	 * This ensures operations trigger udev events which only monitors /dev.
+	 */
+	if (strcmp(_dm_dir, DEV_DIR) != 0) {
+		char real_path[PATH_MAX];
+		struct stat real_stat;
+
+		/* Build path to real /dev node (kernel always creates /dev/dm-N) */
+		if (dm_snprintf(real_path, sizeof(real_path), DEV_DIR "dm-%u", minor) >= 0) {
+			/* Check if real node exists with matching dev */
+			if (stat(real_path, &real_stat) >= 0 &&
+			    S_ISBLK(real_stat.st_mode) &&
+			    real_stat.st_rdev == dev) {
+				/*
+				 * Real /dev/dm-N exists. Create symlink from alternative location.
+				 * This allows operations to work through the symlink and trigger
+				 * udev events on the real device.
+				 */
+				log_debug_activation("Creating symlink %s -> %s", path, real_path);
+				(void) dm_prepare_selinux_context(path, S_IFLNK);
+				if (symlink(real_path, path) < 0) {
+					log_sys_error("symlink", path);
+					(void) dm_prepare_selinux_context(NULL, 0);
+					return 0;
+				}
+				(void) dm_prepare_selinux_context(NULL, 0);
+				log_debug_activation("Created symlink %s -> %s", path, real_path);
+				return 1;
+			}
+		}
+	}
 
 	(void) dm_prepare_selinux_context(path, S_IFBLK);
 	old_mask = umask(0);
@@ -1103,8 +1166,8 @@ static int _rm_dev_node(const char *dev_name, int warn_if_udev_failed)
 			 "Falling back to direct node removal.", path);
 
 	/* udev may already have deleted the node. Ignore ENOENT. */
-	if (unlink(path) < 0 && errno != ENOENT) {
-		log_error("Unable to unlink device node for '%s'", dev_name);
+	if (unlink(path) && (errno != ENOENT)) {
+		log_sys_error("unlink", path);
 		return 0;
 	}
 
@@ -1443,14 +1506,15 @@ struct node_op_parms {
 	char *old_name;
 	int warn_if_udev_failed;
 	unsigned rely_on_udev;
-	char names[];
+	char names[0];
 };
 
 static void _store_str(char **pos, char **ptr, const char *str)
 {
-	strcpy(*pos, str);
+	size_t len = strlen(str) + 1;
+	memcpy(*pos, str, len);
 	*ptr = *pos;
-	*pos += strlen(*ptr) + 1;
+	*pos += len;
 }
 
 static void _del_node_op(struct node_op_parms *nop)
@@ -1700,15 +1764,17 @@ const char *dm_sysfs_dir(void)
  */
 int dm_set_uuid_prefix(const char *uuid_prefix)
 {
+	size_t len;
+
 	if (!uuid_prefix)
 		return_0;
 
-	if (strlen(uuid_prefix) > DM_MAX_UUID_PREFIX_LEN) {
+	if ((len = strlen(uuid_prefix)) > DM_MAX_UUID_PREFIX_LEN) {
 		log_error("New uuid prefix %s too long.", uuid_prefix);
 		return 0;
 	}
 
-	strcpy(_default_uuid_prefix, uuid_prefix);
+	memcpy(_default_uuid_prefix, uuid_prefix, len + 1);
 
 	return 1;
 }
@@ -1737,12 +1803,15 @@ static void _unmangle_mountinfo_string(const char *src, char *buf)
 	*buf = '\0';
 }
 
+/* coverity[+tainted_string_sanitize_content:arg-0] */
+static int _sanitize_line(const char *line) { return 1; }
+
 /* Parse one line of mountinfo and unmangled target line */
 static int _mountinfo_parse_line(const char *line, unsigned *maj, unsigned *min, char *buf)
 {
 	char root[PATH_MAX + 1]; /* sscanf needs extra '\0' */
 	char target[PATH_MAX + 1];
-	char *devmapper;
+	const char *devmapper;
 	struct dm_task *dmt;
 	struct dm_info info;
 	unsigned i;
@@ -1787,7 +1856,7 @@ static int _mountinfo_parse_line(const char *line, unsigned *maj, unsigned *min,
 }
 
 /*
- * Function to operate on individal mountinfo line,
+ * Function to operate on individual mountinfo line,
  * minor, major and mount target are parsed and unmangled
  */
 int dm_mountinfo_read(dm_mountinfo_line_callback_fn read_fn, void *cb_data)
@@ -1807,7 +1876,8 @@ int dm_mountinfo_read(dm_mountinfo_line_callback_fn read_fn, void *cb_data)
 	}
 
 	while (!feof(minfo) && fgets(buffer, sizeof(buffer), minfo))
-		if (!_mountinfo_parse_line(buffer, &maj, &min, target) ||
+		if (!_sanitize_line(buffer) ||
+		    !_mountinfo_parse_line(buffer, &maj, &min, target) ||
 		    !read_fn(buffer, maj, min, target, cb_data)) {
 			stack;
 			r = 0;
@@ -1822,32 +1892,27 @@ int dm_mountinfo_read(dm_mountinfo_line_callback_fn read_fn, void *cb_data)
 
 static int _sysfs_get_dm_name(uint32_t major, uint32_t minor, char *buf, size_t buf_size)
 {
-	char *sysfs_path, *temp_buf = NULL;
+	char sysfs_path[PATH_MAX], temp_buf[2 * DM_NAME_LEN];
 	FILE *fp = NULL;
 	int r = 0;
 	size_t len;
 
-	if (!(sysfs_path = dm_malloc(PATH_MAX)) ||
-	    !(temp_buf = dm_malloc(PATH_MAX))) {
-		log_error("_sysfs_get_dm_name: failed to allocate temporary buffers");
-		goto bad;
-	}
-
-	if (dm_snprintf(sysfs_path, PATH_MAX, "%sdev/block/%" PRIu32 ":%" PRIu32
+	if (dm_snprintf(sysfs_path, sizeof(sysfs_path),
+			"%sdev/block/%" PRIu32 ":%" PRIu32
 			"/dm/name", _sysfs_dir, major, minor) < 0) {
-		log_error("_sysfs_get_dm_name: dm_snprintf failed");
+		log_error("_sysfs_get_dm_name: dm_snprintf failed.");
 		goto bad;
 	}
 
 	if (!(fp = fopen(sysfs_path, "r"))) {
-		if (errno != ENOENT)
-			log_sys_error("fopen", sysfs_path);
-		else
+		if (errno == ENOENT)
 			log_sys_debug("fopen", sysfs_path);
+                else
+			log_sys_error("fopen", sysfs_path);
 		goto bad;
 	}
 
-	if (!fgets(temp_buf, PATH_MAX, fp)) {
+	if (!fgets(temp_buf, sizeof(temp_buf), fp)) {
 		log_sys_error("fgets", sysfs_path);
 		goto bad;
 	}
@@ -1855,20 +1920,21 @@ static int _sysfs_get_dm_name(uint32_t major, uint32_t minor, char *buf, size_t
 	len = strlen(temp_buf);
 
 	if (len > buf_size) {
-		log_error("_sysfs_get_dm_name: supplied buffer too small");
+		log_error("_sysfs_get_dm_name: supplied buffer too small.");
 		goto bad;
 	}
 
-	temp_buf[len ? len - 1 : 0] = '\0'; /* \n */
-	strcpy(buf, temp_buf);
+	if (len)
+		--len;  /* strip \n */
+
+	memcpy(buf, temp_buf, len);
+	buf[len] = '\0';
+
 	r = 1;
 bad:
 	if (fp && fclose(fp))
 		log_sys_error("fclose", sysfs_path);
 
-	dm_free(temp_buf);
-	dm_free(sysfs_path);
-
 	return r;
 }
 
@@ -1952,7 +2018,7 @@ static int _sysfs_find_kernel_name(uint32_t major, uint32_t minor, char *buf, si
 				    !strcmp(name_dev, "holders") ||
 				    !strcmp(name_dev, "integrity") ||
 				    !strcmp(name_dev, "loop") ||
-				    !strcmp(name_dev, "queueu") ||
+				    !strcmp(name_dev, "queue") ||
 				    !strcmp(name_dev, "md") ||
 				    !strcmp(name_dev, "mq") ||
 				    !strcmp(name_dev, "power") ||
@@ -1989,31 +2055,25 @@ static int _sysfs_find_kernel_name(uint32_t major, uint32_t minor, char *buf, si
 
 static int _sysfs_get_kernel_name(uint32_t major, uint32_t minor, char *buf, size_t buf_size)
 {
-	char *name, *sysfs_path, *temp_buf = NULL;
+	char *name;
+	char sysfs_path[PATH_MAX], temp_buf[PATH_MAX];
 	ssize_t size;
-	size_t len;
 	int r = 0;
 
-	if (!(sysfs_path = dm_malloc(PATH_MAX)) ||
-	    !(temp_buf = dm_malloc(PATH_MAX))) {
-		log_error("_sysfs_get_kernel_name: failed to allocate temporary buffers");
-		goto bad;
-	}
-
-	if (dm_snprintf(sysfs_path, PATH_MAX, "%sdev/block/%" PRIu32 ":%" PRIu32,
+	if (dm_snprintf(sysfs_path, sizeof(sysfs_path),
+			"%sdev/block/%" PRIu32 ":%" PRIu32,
 			_sysfs_dir, major, minor) < 0) {
 		log_error("_sysfs_get_kernel_name: dm_snprintf failed");
 		goto bad;
 	}
 
-	if ((size = readlink(sysfs_path, temp_buf, PATH_MAX - 1)) < 0) {
-		if (errno != ENOENT)
-			log_sys_error("readlink", sysfs_path);
-		else {
+	if ((size = readlink(sysfs_path, temp_buf, sizeof(temp_buf) - 1)) < 0) {
+		if (errno == ENOENT) {
 			log_sys_debug("readlink", sysfs_path);
 			r = _sysfs_find_kernel_name(major, minor, buf, buf_size);
 			goto out;
-		}
+		} else
+			log_sys_error("readlink", sysfs_path);
 		goto bad;
 	}
 	temp_buf[size] = '\0';
@@ -2022,20 +2082,15 @@ static int _sysfs_get_kernel_name(uint32_t major, uint32_t minor, char *buf, siz
 		log_error("Could not locate device kernel name in sysfs path %s", temp_buf);
 		goto bad;
 	}
-	name += 1;
-	len = size - (name - temp_buf) + 1;
 
-	if (len > buf_size) {
+	if (!_dm_strncpy(buf, name + 1, buf_size)) {
 		log_error("_sysfs_get_kernel_name: output buffer too small");
 		goto bad;
 	}
 
-	strcpy(buf, name);
 	r = 1;
 bad:
 out:
-	dm_free(temp_buf);
-	dm_free(sysfs_path);
 
 	return r;
 }
@@ -2309,7 +2364,7 @@ static int _check_semaphore_is_supported(void)
 
 	if (maxid < 0) {
 		log_warn("Kernel not configured for semaphores (System V IPC). "
-			 "Not using udev synchronisation code.");
+			 "Not using udev synchronization code.");
 		return 0;
 	}
 
@@ -2332,7 +2387,7 @@ static int _check_udev_is_running(void)
 
 	if (!(r = udev_queue_get_udev_is_active(udev_queue)))
 		log_debug_activation("Udev is not running. "
-				     "Not using udev synchronisation code.");
+				     "Not using udev synchronization code.");
 
 	udev_queue_unref(udev_queue);
 	udev_unref(udev);
@@ -2407,7 +2462,7 @@ static int _get_cookie_sem(uint32_t cookie, int *semid)
 			break;
 		case EACCES:
 			log_error("No permission to access "
-				  "notificaton semaphore identified "
+				  "notification semaphore identified "
 				  "by cookie value %" PRIu32 " (0x%x)",
 				  cookie, cookie);
 			break;
@@ -2428,20 +2483,20 @@ static int _udev_notify_sem_inc(uint32_t cookie, int semid)
 	int val;
 
 	if (semop(semid, &sb, 1) < 0) {
-		log_error("semid %d: semop failed for cookie 0x%" PRIx32 ": %s",
+		log_error("cookie inc: semid %d: semop failed for cookie 0x%" PRIx32 ": %s",
 			  semid, cookie, strerror(errno));
 		return 0;
 	}
 
  	if ((val = semctl(semid, 0, GETVAL)) < 0) {
-		log_error("semid %d: sem_ctl GETVAL failed for "
+		log_warn("cookie inc: semid %d: sem_ctl GETVAL failed for "
 			  "cookie 0x%" PRIx32 ": %s",
 			  semid, cookie, strerror(errno));
-		return 0;		
-	}
-
-	log_debug_activation("Udev cookie 0x%" PRIx32 " (semid %d) incremented to %d",
-		  cookie, semid, val);
+		log_debug_activation("Udev cookie 0x%" PRIx32 " (semid %d) incremented.",
+				      cookie, semid);
+	} else
+		log_debug_activation("Udev cookie 0x%" PRIx32 " (semid %d) incremented to %d",
+				     cookie, semid, val);
 
 	return 1;
 }
@@ -2451,23 +2506,21 @@ static int _udev_notify_sem_dec(uint32_t cookie, int semid)
 	struct sembuf sb = {0, -1, IPC_NOWAIT};
 	int val;
 
- 	if ((val = semctl(semid, 0, GETVAL)) < 0) {
-		log_error("semid %d: sem_ctl GETVAL failed for "
-			  "cookie 0x%" PRIx32 ": %s",
-			  semid, cookie, strerror(errno));
-		return 0;
-	}
+ 	if ((val = semctl(semid, 0, GETVAL)) < 0)
+		log_warn("cookie dec: semid %d: sem_ctl GETVAL failed for "
+			 "cookie 0x%" PRIx32 ": %s",
+			 semid, cookie, strerror(errno));
 
 	if (semop(semid, &sb, 1) < 0) {
 		switch (errno) {
 			case EAGAIN:
-				log_error("semid %d: semop failed for cookie "
+				log_error("cookie dec: semid %d: semop failed for cookie "
 					  "0x%" PRIx32 ": "
 					  "incorrect semaphore state",
 					  semid, cookie);
 				break;
 			default:
-				log_error("semid %d: semop failed for cookie "
+				log_error("cookie dec: semid %d: semop failed for cookie "
 					  "0x%" PRIx32 ": %s",
 					  semid, cookie, strerror(errno));
 				break;
@@ -2475,9 +2528,12 @@ static int _udev_notify_sem_dec(uint32_t cookie, int semid)
 		return 0;
 	}
 
-	log_debug_activation("Udev cookie 0x%" PRIx32 " (semid %d) decremented to %d",
-			     cookie, semid, val - 1);
-
+	if (val < 0)
+		log_debug_activation("Udev cookie 0x%" PRIx32 " (semid %d) decremented.",
+				     cookie, semid);
+	else
+		log_debug_activation("Udev cookie 0x%" PRIx32 " (semid %d) decremented to %d",
+				     cookie, semid, val - 1);
 	return 1;
 }
 
@@ -2554,7 +2610,7 @@ static int _udev_notify_sem_create(uint32_t *cookie, int *semid)
 	sem_arg.val = 1;
 
 	if (semctl(gen_semid, 0, SETVAL, sem_arg) < 0) {
-		log_error("semid %d: semctl failed: %s", gen_semid, strerror(errno));
+		log_error("cookie create: semid %d: semctl failed: %s", gen_semid, strerror(errno));
 		/* We have to destroy just created semaphore
 		 * so it won't stay in the system. */
 		(void) _udev_notify_sem_destroy(gen_cookie, gen_semid);
@@ -2562,9 +2618,10 @@ static int _udev_notify_sem_create(uint32_t *cookie, int *semid)
 	}
 
  	if ((val = semctl(gen_semid, 0, GETVAL)) < 0) {
-		log_error("semid %d: sem_ctl GETVAL failed for "
+		log_error("cookie create: semid %d: sem_ctl GETVAL failed for "
 			  "cookie 0x%" PRIx32 ": %s",
 			  gen_semid, gen_cookie, strerror(errno));
+		(void) _udev_notify_sem_destroy(gen_cookie, gen_semid);
 		goto bad;
 	}
 
