@@ -12,9 +12,8 @@
 
 test_description='Set up things to run tests with dlm'
 
-. lib/inittest
 
-[ -z "$LVM_TEST_LOCK_TYPE_DLM" ] && skip;
+. lib/inittest --skip-with-lvmpolld --with-lock-type-dlm
 
 aux prepare_dlm
 aux prepare_lvmlockd
