@@ -43,6 +43,9 @@ arg(activationmode_ARG, '\0', "activationmode", activationmode_VAL, 0, 0,
 arg(adddev_ARG, '\0', "adddev", pv_VAL, 0, 0,
     "Add a device to the devices file.\n")
 
+arg(addid_ARG, '\0', "addid", string_VAL, 0, 0,
+    "Find the device with the given device_id and add it to the devices file.\n")
+
 arg(addpvid_ARG, '\0', "addpvid", string_VAL, 0, 0,
     "Find a device with the PVID and add the device to the devices file.\n")
 
@@ -213,6 +216,9 @@ arg(deldev_ARG, '\0', "deldev", string_VAL, 0, 0,
     "When used alone, --deldev specifies a device name.\n"
     "When used with --deviceidtype, --deldev specifies a device id.\n")
 
+arg(delid_ARG, '\0', "delid", string_VAL, 0, 0,
+    "Remove the device with the specified device ID from the devices file.\n")
+
 arg(delnotfound_ARG, '\0', "delnotfound", 0, 0, 0,
     "Remove devices file entries with no matching device.\n")
 
@@ -381,6 +387,9 @@ arg(labelsector_ARG, '\0', "labelsector", number_VAL, 0, 0,
     "start of the disk (between 0 and 3 inclusive - see LABEL_SCAN_SECTORS\n"
     "in the source). Use with care.\n")
 
+arg(listids_ARG, '\0', "listids", pv_VAL, 0, 0,
+    "Print a list of device IDs available for the device.\n")
+
 arg(listlvs_ARG, '\0', "listlvs", 0, 0, 0,
     "Print a list of LVs that use the device.\n")
 
@@ -786,6 +795,20 @@ arg(setautoactivation_ARG, '\0', "setautoactivation", bool_VAL, 0, 0,
     "If autoactivation is enabled on a VG, autoactivation can be disabled\n"
     "for individual LVs.\n")
 
+arg(setlockargs_ARG, '\0', "setlockargs", string_VAL, 0, 0,
+    "Add or remove lock_args settings for a shared VG.\n"
+    "The lock_args determine lock manager behavior for the VG.\n"
+    "These settings are only allowed for lock_type sanlock.\n"
+    "persist: use persistent reservations for lock recovery.\n"
+    "lvmlockd will preempt-abort the persistent reservation of a failed\n"
+    "lock owner so that the lock can be acquired.\n"
+    "notimeout: use locks that do not time out when the owner fails.\n"
+    "In this case, a lock owned by a failed host can only be acquired\n"
+    "using the persist feature.\n"
+    "nopersist: do not use the persist feature.\n"
+    "timeout: do not use the notimeout feature.\n"
+    "The default behavior with no settings configured is: nopersist and timeout.\n")
+
 arg(setpersist_ARG, '\0', "setpersist", string_VAL, 0, 0,
     "#vgcreate\n"
     "Set flags to control persistent reservation behavior.\n"
@@ -904,7 +927,7 @@ arg(syncaction_ARG, '\0', "syncaction", syncaction_VAL, 0, 0,
     "Mind that these synchronization actions are transient and have to be restarted\n"
     "after a system failure/reboot or a configuration change to the RaidLV.\n"
     "See \\fBlvs\\fP(8) for reporting discrepancies found or repaired.\n")
-    
+
 arg(sysinit_ARG, '\0', "sysinit", 0, 0, 0,
     "Indicates that vgchange/lvchange is being invoked\n"
     "from early system initialisation\n"
@@ -1379,7 +1402,7 @@ arg(file_ARG, 'f', "file", string_VAL, 0, 0,
     "When backing up more than one VG, the file name is\n"
     "treated as a template, and %s is replaced by the VG name.\n"
     "#vgcfgrestore\n"
-    "Read metadata backup from the named file.\n" 
+    "Read metadata backup from the named file.\n"
     "Usually this file was created by vgcfgbackup.\n")
 
 arg(force_ARG, 'f', "force", 0, ARG_COUNTABLE, 0,
@@ -1497,8 +1520,8 @@ arg(list_ARG, 'l', "list", 0, 0, 0,
     "#lvmconfig\n"
     "#dumpconfig\n"
     "#config\n"
-    "List config settings with summarizing comment. This is the same as using\n"
-    "options --typeconfig list --withsummary.\n"
+    "Print a list of config settings instead of structured config settings tree.\n"
+    "If used without other options, this is the same as --typeconfig list --withsummary.\n"
     "#vgcfgrestore\n"
     "List metadata backup and archive files pertaining to the VG.\n"
     "May be used with --file. Does not restore the VG.\n"
@@ -1636,9 +1659,8 @@ arg(options_ARG, 'o', "options", string_VAL, ARG_GROUPABLE, 0,
     "Use field name \\fBlv_all\\fP to view all LV fields,\n"
     "\\fBvg_all\\fP all VG fields,\n"
     "\\fBpv_all\\fP all PV fields,\n"
-    "\\fBpvseg_all\\fP all PV segment fields,\n"
     "\\fBseg_all\\fP all LV segment fields, and\n"
-    "\\fBpvseg_all\\fP all PV segment columns.\n"
+    "\\fBpvseg_all\\fP all PV segment fields.\n"
     "See the \\fBlvm.conf\\fP(5) report section for more config options.\n"
     "See \\fBlvmreport\\fP(7) for more information about reporting.\n")
 
